(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[931],{53963:function(e,t,s){Promise.resolve().then(s.bind(s,68157))},16463:function(e,t,s){"use strict";var n=s(71169);s.o(n,"usePathname")&&s.d(t,{usePathname:function(){return n.usePathname}}),s.o(n,"useRouter")&&s.d(t,{useRouter:function(){return n.useRouter}}),s.o(n,"useSearchParams")&&s.d(t,{useSearchParams:function(){return n.useSearchParams}})},68157:function(e,t,s){"use strict";s.r(t),s.d(t,{default:function(){return a}});var n=s(57437),l=s(16463);function a(){let e=(0,l.useRouter)();return(0,n.jsx)("div",{className:"relative bg-white overflow-hidden",children:(0,n.jsx)("div",{className:"max-w-7xl mx-auto",children:(0,n.jsxs)("div",{className:"relative z-10 pb-8 bg-white sm:pb-16 md:pb-20 lg:max-w-2xl lg:w-full lg:pb-28 xl:pb-32",children:[(0,n.jsx)("svg",{className:"hidden lg:block absolute right-0 inset-y-0 h-full w-48 text-white transform translate-x-1/2",fill:"currentColor",viewBox:"0 0 100 100",preserveAspectRatio:"none","aria-hidden":"true",children:(0,n.jsx)("polygon",{points:"50,0 100,0 50,100 0,100"})}),(0,n.jsx)("main",{className:"mt-10 mx-auto max-w-7xl px-4 sm:mt-12 sm:px-6 md:mt-16 lg:mt-20 lg:px-8 xl:mt-28",children:(0,n.jsxs)("div",{className:"sm:text-center lg:text-left",children:[(0,n.jsx)("h1",{className:"text-4xl tracking-tight font-bold text-gray-900 sm:text-5xl sm:tracking-tight md:text-6xl md:tracking-tight",children:(0,n.jsx)("span",{className:"block xl:inline",children:"Dbt Column Lineage"})}),(0,n.jsx)("p",{className:"mt-3 text-base text-gray-500 sm:mt-5 sm:text-lg sm:max-w-xl sm:mx-auto md:mt-5 md:text-xl lg:mx-0",children:"dbt column lineage visualization tool for data analysts and engineers."}),(0,n.jsx)("div",{className:"mt-5 sm:mt-8 sm:flex sm:justify-center lg:justify-start",children:(0,n.jsx)("button",{onClick:()=>e.push("/cl"),type:"button",className:"text-white bg-blue-400 hover:bg-blue-500 focus:ring-4 focus:outline-none font-medium rounded-lg text-sm px-5 py-2.5 text-center mr-2 mb-2",children:"Try it out"})})]})})]})})})}}},function(e){e.O(0,[971,23,744],function(){return e(e.s=53963)}),_N_E=e.O()}]);